@@ -309,6 +309,18 @@ static void * op_to_func(MVMThreadContext *tc, MVMint16 opcode) {
     case MVM_OP_randscale_n: return &MVM_proc_randscale_n;
     case MVM_OP_isnanorinf: return &MVM_num_isnanorinf;
     case MVM_OP_nativecallinvoke: return &MVM_nativecall_invoke;
+    case MVM_OP_iscont_i: return &MVM_6model_container_iscont_i;
+    case MVM_OP_iscont_n: return &MVM_6model_container_iscont_n;
+    case MVM_OP_iscont_s: return &MVM_6model_container_iscont_s;
+    case MVM_OP_assign_i: return &MVM_6model_container_assign_i;
+    case MVM_OP_assign_n: return &MVM_6model_container_assign_n;
+    case MVM_OP_assign_s: return &MVM_6model_container_assign_s;
+    case MVM_OP_decont_i: return &MVM_6model_container_decont_i;
+    case MVM_OP_decont_n: return &MVM_6model_container_decont_n;
+    case MVM_OP_decont_s: return &MVM_6model_container_decont_s;
+    case MVM_OP_getlexref_i: return &MVM_nativeref_lex_i;
+    case MVM_OP_getlexref_n: return &MVM_nativeref_lex_n;
+    case MVM_OP_getlexref_s: return &MVM_nativeref_lex_s;
     case MVM_OP_sp_boolify_iter: return &MVM_iter_istrue;
     case MVM_OP_prof_allocated: return &MVM_profile_log_allocated;
     case MVM_OP_prof_exit: return &MVM_profile_log_exit;
@@ -1096,7 +1108,6 @@ static MVMint32 jgb_consume_ins(MVMThreadContext *tc, JitGraphBuilder *jgb,
     }
     case MVM_OP_bindpos_i:
     case MVM_OP_bindpos_o:
-    case MVM_OP_bindpos_n:
     case MVM_OP_bindpos_s:
     case MVM_OP_bindkey_o: {
         MVMint32 invocant = ins->operands[0].reg.orig;
@@ -1109,6 +1120,17 @@ static MVMint32 jgb_consume_ins(MVMThreadContext *tc, JitGraphBuilder *jgb,
         jgb_append_call_c(tc, jgb, op_to_func(tc, op), 4, args, MVM_JIT_RV_VOID, -1);
         break;
     }
+    case MVM_OP_bindpos_n: {
+        MVMint32 invocant = ins->operands[0].reg.orig;
+        MVMint32 pos = ins->operands[1].reg.orig;
+        MVMint32 value = ins->operands[2].reg.orig;
+        MVMJitCallArg args[] = { { MVM_JIT_INTERP_VAR, MVM_JIT_INTERP_TC },
+                                 { MVM_JIT_REG_VAL, invocant },
+                                 { MVM_JIT_REG_VAL, pos },
+                                 { MVM_JIT_REG_VAL_F, value } };
+        jgb_append_call_c(tc, jgb, op_to_func(tc, op), 4, args, MVM_JIT_RV_VOID, -1);
+        break;
+    }
     case MVM_OP_getattr_i:
     case MVM_OP_getattr_n:
     case MVM_OP_getattr_s:
@@ -1684,6 +1706,51 @@ static MVMint32 jgb_consume_ins(MVMThreadContext *tc, JitGraphBuilder *jgb,
         jgb_append_call_c(tc, jgb, op_to_func(tc, op), 4, args,
                           MVM_JIT_RV_PTR, dst);
         break;
+    }
+        /* native references (as simple function calls for now) */
+    case MVM_OP_iscont_i:
+    case MVM_OP_iscont_n:
+    case MVM_OP_iscont_s: {
+        MVMint16 dst = ins->operands[0].reg.orig;
+        MVMint16 obj = ins->operands[1].reg.orig;
+        MVMJitCallArg args[] = { { MVM_JIT_INTERP_VAR, MVM_JIT_INTERP_TC },
+                                 { MVM_JIT_REG_VAL, obj } };
+        jgb_append_call_c(tc, jgb, op_to_func(tc, op), 2, args, MVM_JIT_RV_INT, dst);
+        break;
+    }
+    case MVM_OP_assign_i:
+    case MVM_OP_assign_n:
+    case MVM_OP_assign_s: {
+        MVMint16 target = ins->operands[0].reg.orig;
+        MVMint16 value  = ins->operands[1].reg.orig;
+        MVMJitCallArg args[] = { { MVM_JIT_INTERP_VAR, MVM_JIT_INTERP_TC },
+                                 { MVM_JIT_REG_VAL, target },
+                                 { MVM_JIT_REG_VAL, value } };
+        jgb_append_call_c(tc, jgb, op_to_func(tc, op), 3, args, MVM_JIT_RV_VOID, -1);
+        break;
+    }
+    case MVM_OP_decont_i:
+    case MVM_OP_decont_n:
+    case MVM_OP_decont_s: {
+        MVMint16 dst = ins->operands[0].reg.orig;
+        MVMint16 obj = ins->operands[1].reg.orig;
+        MVMJitCallArg args[] = { { MVM_JIT_INTERP_VAR, MVM_JIT_INTERP_TC },
+                                 { MVM_JIT_REG_VAL, obj },
+                                 { MVM_JIT_REG_ADDR, dst } };
+        jgb_append_call_c(tc, jgb, op_to_func(tc, op), 3, args, MVM_JIT_RV_VOID, -1);
+        break;
+    }
+    case MVM_OP_getlexref_i:
+    case MVM_OP_getlexref_n:
+    case MVM_OP_getlexref_s: {
+        MVMint16 dst     = ins->operands[0].reg.orig;
+        MVMuint16 outers = ins->operands[1].lex.outers;
+        MVMuint16 idx    = ins->operands[1].lex.idx;
+        MVMJitCallArg args[] = { { MVM_JIT_INTERP_VAR, MVM_JIT_INTERP_TC },
+                                 { MVM_JIT_LITERAL, outers },
+                                 { MVM_JIT_LITERAL, idx } };
+        jgb_append_call_c(tc, jgb, op_to_func(tc, op), 3, args, MVM_JIT_RV_PTR, dst);
+        break;
     }
         /* profiling */
     case MVM_OP_prof_allocated: {
